require 'csv'

module Tolq::Parsers::CSV
  # Unparses a tolq response to CSV suitable
  class Unparser
    # Unparses the tolq response
    #
    # @param tolq_response [Hash] A parsed response from our api
    # @return [String] A csv formatted string
    def from_tolq_response(tolq_response, **args)
      translations = tolq_response['translations'].values.first
      generate_csv(translations)
    end

    # Unparses a translations hash of key => translation
    #
    # @param translations [Hash] A hash of translations with key => translation
    # @return [String] A csv formatted string
    def from_hash(translations, **args)
      generate_csv(translations)
    end

    private

    def generate_csv(translations)
      CSV.generate(force_quotes: true) do |csv|
        build_translation_matrix(translations).each do |row|
          if row.nil? # skipped empty rows
            csv << [].fill(nil, 0..@maxcol)
          else
            csv << row.fill(nil, row.length..@maxcol)
          end
        end
      end
    end

    def build_translation_matrix(translations_hash)
      @maxcol = 0
      translations_hash.inject([]) do |matrix, (k,translation)|
        row,col = ColumnHelper.from_char_notation(k)
        matrix[row] ||= []
        matrix[row][col] = translation
        @maxcol = col if col > @maxcol
        matrix
      end
    end
  end
end
